#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
#  Configuration
#  -------------
#
#  1. Define a new keyboard of type 'i2c'
#  2. Define inputPins section for that keyboard
#
#  Sample:
#
#  [keyboards]
#  arduino = i2c
#
#  [arduino_keyboard]
#  delete_key = D #key which clears the entered code - default D
#  dismisstime = 10 #time in seconds after the entered code is cleared - default 10
#  i2cadress = 0x04 #i2c adress of the arduino, default 4
#  port = 1 # local i2c port: 0 = /dev/i2c-0 (port I2C0), 1 = /dev/i2c-1 (port I2C1) - default 1
#  register = I2C register (not used in the sketch, default 1)
#  chars = 0,1,2,3,4,5,6,7,8,9,A,B,C,D,*,# #chars accepted by keyboard default 0,1,2,3,4,5,6,7,8,9,A,B,C,D,*,#
#  maxretries = 10 # how often should we retry sending to I2C before giving up? default 10
#
#  [arduino_InputPins]
#  1234 = out:ArduinoTerminal,10,110,3 # sends 10 to arduino when code 1234 is entered, waits 3 seconds and sends 110
#
#  [arduino_OutputPins]
#  SEND = ArduinoTerminal #needed
#
#  That's all...
#
#  Hardware Connections
#  --------------------
#  The keyboard is written specifically to read a matrix-style keyboard
#  connected to a arduino and to switch a relay which is also connected to the same arduino
#  Any adaptions should be easy to do. Please also have a look at the sketch at ......
#
#  Connect the I2C Pins of Arduino and Raspberry as follows:
#  SDA (A4 on an UNO) to SCL (GPIO3, Pin 5) on the Raspi
#  SCL (A5 on an UNO) to SDA (GPIO2, Pin 3) on the Raspi
#  GND of Arduino to GND of Raspi
#  see also http://blog.oscarliang.net/raspberry-pi-arduino-connected-i2c/
#  IMPORTANT  --- IMPORTANT  ---- IMPORTANT  --- IMPORTANT  ---- IMPORTANT  --- IMPORTANT
#  you will also need the smbus-libs for python
#  to install on raspbian simply do
#  sudo apt-get install python-smbus
#

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import threading
import serial
import time

from doorpi.keyboard.AbstractBaseClass import KeyboardAbstractBaseClass, HIGH_LEVEL, LOW_LEVEL
import doorpi
import smbus

def get(**kwargs): return I2C(**kwargs)
class I2C(KeyboardAbstractBaseClass):
    name = 'I2C basic keyboard and relay keyboard'

    def readtast(self):
        try:
            tast = self._bus.read_byte_data(self.__address, self.__register)
            if chr(tast) in self.__allow_chars:
                logger.debug("I2C:" + chr(tast))
                return tast
            else:
                return None
        except IOError:
            return None

    def sendi2c(self, pin):
        try:
            self._bus.write_byte(self.__address, int(pin))
            return True
        except IOError:
            return False

    def i_to_c(self):
        try:
            chars = ""
            while not self._shutdown:
                now = time.time()
                newchar = self.readtast()
                if newchar is not None and newchar != None:
                    # löschen wenn delete_key gedrückt wurde
                    if chr(newchar) == self.__delete_key:
                        logger.debug("delete")
                        chars = ""
                        continue
                    logger.debug("I2C new char %s read", newchar)
                    logger.debug("now %s  - lastkey %s", now, self.last_key_time)
                    #string loeschen, wenn innerhalb der festgelegten zeit keine taste gedrückt wurde
                    if now - self.last_key_time > self.__dismisstime:
                        chars = ""
                    chars += chr(newchar)
                    self.last_key_time = now
                    if chars in self._InputPins:
                        self.last_key = chars
                        self._fire_OnKeyDown(self.last_key, __name__)
                        self._fire_OnKeyPressed(self.last_key, __name__)
                        doorpi.DoorPi().event_handler('OnFoundKnownTag', __name__)
                        self._fire_OnKeyUp(self.last_key, __name__)
                        logger.debug("key is %s", self.last_key)
                        chars = ""
        except Exception as ex:
            logger.exception(ex)
        finally:
            # shutdown the UART
            logger.debug("I2C thread ended")

    def __init__(self, input_pins, output_pins, keyboard_name, conf_pre, conf_post, *args, **kwargs):
        logger.debug("__init__ (input_pins = %s)", input_pins)
        self.keyboard_name = keyboard_name
        self.last_key = ""
        self.last_key_time = 0
        # auslesen aus ini:
        section_name = conf_pre+'keyboard'+conf_post
        self.__dismisstime = doorpi.DoorPi().config.get_int(section_name, 'dismisstime', 20)
        self.__delete_key = doorpi.DoorPi().config.get_string_parsed(section_name, 'delete_key', 'D')
        self.__address = int(doorpi.DoorPi().config.get_string_parsed(section_name, 'i2c_address', 4))
        self.__register = int(doorpi.DoorPi().config.get_string_parsed(section_name, 'i2c_register', 1))
        self.__port = int(doorpi.DoorPi().config.get_string_parsed(section_name, 'port', 1))
        self.__maxretries = int(doorpi.DoorPi().config.get_string_parsed(section_name, 'maxretries', 10))
        self.__allow_chars = doorpi.DoorPi().config.get_string_parsed(
            section_name, 'chars', '0,1,2,3,4,5,6,7,8,9,A,B,C,D,*,#')
        self.__allow_chars = self.__allow_chars.split(',')
        self._InputPins = map(str, input_pins)
        for input_pin in self._InputPins:
           self._register_EVENTS_for_pin(input_pin, __name__)
        self._OutputPins = map(str, output_pins)
        for output_pin in self._OutputPins:
           self._register_EVENTS_for_pin(output_pin, __name__)
        self._bus = smbus.SMBus(self.__port)

        doorpi.DoorPi().event_handler.register_event('OnFoundKnownTag', __name__)

        self._shutdown = False
        self._thread = threading.Thread(target = self.i_to_c)
        self._thread.daemon = True
        self._thread.start()
        self.register_destroy_action()

    def destroy(self):
        if self.is_destroyed: return
        logger.debug("destroy")
        self._shutdown = True
        doorpi.DoorPi().event_handler.unregister_source(__name__, True)
        self.__destroyed = True

    def status_input(self, tag):
        logger.debug("status_input for tag %s", tag)
        if tag == self.last_key:
            return True
        else:
            return False

    def set_output(self, pin, value, log_output = True):
        for x in xrange(1, self.__maxretries):
            sent = self.sendi2c(value)
            if sent:
                logger.debug('Sent %s to I2C', pin)
                return True
        if x == 11:
            logger.debug('Error: Sending %s to I2C failed! Giving up!', pin)
            return False
